package com.bocloud.mcs.collector.cisco;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.snmp4j.CommunityTarget;
import org.snmp4j.PDU;
import org.snmp4j.Snmp;
import org.snmp4j.mp.SnmpConstants;
import org.snmp4j.smi.*;
import org.snmp4j.util.DefaultPDUFactory;
import org.snmp4j.util.TableEvent;
import org.snmp4j.util.TableUtils;
import java.util.*;

public class SNMPv2Utils {
    private static final Logger logger = LoggerFactory.getLogger(SNMPv2Utils.class);
    private static final String DEFAULT_PROTOCOL = "udp";
    private static final long DEFAULT_TIMEOUT = 3 * 1000L;
    private static final int DEFAULT_RETRY = 3;

    /**
     * 创建target对象
     *
     * @param ip
     * @param community
     * @return
     */
    public static CommunityTarget createDefault(String ip, String community,Integer port) {
        if (StringUtils.isBlank(ip)) {
            throw new NullPointerException("ip is null.");
        }

        if (StringUtils.isBlank(community)) {
            throw new NullPointerException("community is null.");
        }

        Address address = GenericAddress.parse(DEFAULT_PROTOCOL + ":" + ip
                + "/" + port);
        CommunityTarget target = new CommunityTarget();
        target.setCommunity(new OctetString(community));
        target.setAddress(address);
        target.setVersion(SnmpConstants.version2c);
        target.setTimeout(DEFAULT_TIMEOUT);
        target.setRetries(DEFAULT_RETRY);
        return target;
    }

    public static  HashMap<String, Map<String, String>> snmpWalk(Snmp snmp, CommunityTarget target, List<String> OIDStrings) {
        TableUtils utils = new TableUtils(snmp, new DefaultPDUFactory(PDU.GETBULK));//GETNEXT or GETBULK
        utils.setMaxNumRowsPerPDU(1000);   //only for GETBULK, set max-repetitions, default is 10
        OID[] columnOids = new OID[OIDStrings.size()];
        for (int i = 0; i < OIDStrings.size(); i++) {
            columnOids[i] = new OID(OIDStrings.get(i));
        }
        List<TableEvent> l = utils.getTable(target, columnOids, null, null);
        HashMap<String, Map<String, String>> map=new HashMap<>();
        for (TableEvent tableEvent : l) {
            if (tableEvent.isError()) {
                logger.error("send snmpWalk(index:" +tableEvent.getIndex() + ") error:" +tableEvent.getErrorMessage());
            }
            VariableBinding[] columns = tableEvent.getColumns();
            for (VariableBinding vb : columns) {
                Map<String, String> subMap = map.getOrDefault(tableEvent.getIndex().toString(), new HashMap<String, String>());
                if (vb != null) {
                    subMap.put(vb.getOid().toString(), vb.toValueString());
                    map.put(tableEvent.getIndex().toString(), subMap);
                }
            }
        }
        return map;
    }
}
