package com.bocloud.mcs.collector.smartx;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.encrypt.DESEncrptor;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.Common;
import com.bocloud.ims.entity.CloudVendor;
import com.bocloud.ims.enums.CloudProvider;
import com.bocloud.mcs.collector.BocloudCollector;
import com.bocloud.mcs.collector.smartx.common.*;
import com.bocloud.service.core.CurrentService;
import io.prometheus.client.GaugeMetricFamily;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.util.*;

/**
 * Smart-x云监控
 *
 * @author fanzhenxi
 */
@Component
public class SmartXCloudCollector extends BocloudCollector implements InitializingBean {

    private static final Logger logger = LoggerFactory.getLogger(SmartXCloudCollector.class);
    private final static String PREFIX = "smartx_";
    Map<String, Object> cache = new HashMap<>();
    public final static Map<String, String> METRIC_NAME_MAP = new HashMap();

    @Autowired
    private CurrentService currentService;

    static {
        // 主机
        METRIC_NAME_MAP.put("zbs_chunk_readwrite_iops", "主机IOPS总");
        METRIC_NAME_MAP.put("zbs_chunk_read_iops", "主机IOPS读");
        METRIC_NAME_MAP.put("zbs_chunk_write_iops", "主机IOPS写");
        METRIC_NAME_MAP.put("zbs_chunk_readwrite_speed_bps", "主机I/O总带宽");
        METRIC_NAME_MAP.put("zbs_chunk_read_speed_bps", "主机I/O读带宽");
        METRIC_NAME_MAP.put("zbs_chunk_write_speed_bps", "主机I/O写带宽");
        METRIC_NAME_MAP.put("zbs_chunk_avg_readwrite_latency_ns", "主机I/O总平均延时");
        METRIC_NAME_MAP.put("zbs_chunk_avg_read_latency_ns", "主机I/O平均读延时");
        METRIC_NAME_MAP.put("zbs_chunk_avg_write_latency_ns", "主机I/O平均写延时");
        METRIC_NAME_MAP.put("zbs_chunk_cache_capacity_bytes", "主机总缓存容量");
        METRIC_NAME_MAP.put("zbs_chunk_data_capacity_bytes", "主机总数居容量");
        METRIC_NAME_MAP.put("host_ssd_overall_size_bytes", "主机总SSD容量");
        METRIC_NAME_MAP.put("host_hdd_overall_size_bytes", "主机总HDD容量");
        METRIC_NAME_MAP.put("host_cpu_overall_usage_percent", "主机CPU利用率%");
        METRIC_NAME_MAP.put("elf_host_vcpus_provisioned", "主机分配给虚拟机的逻辑核数");
        METRIC_NAME_MAP.put("elf_host_memory_provisioned_bytes", "主机分配给虚拟机的内存大小");
        METRIC_NAME_MAP.put("host_cpu_overall_used_hz", "主机CPU使用频率HZ");
        METRIC_NAME_MAP.put("host_cpu_overall_1m_avg_load", "主机CPU1分钟平均负载");
        METRIC_NAME_MAP.put("host_cpu_overall_5m_avg_load", "主机CPU5分钟平均负载");
        METRIC_NAME_MAP.put("host_cpu_overall_15m_avg_load", "主机CPU15分钟平均负载");
        METRIC_NAME_MAP.put("host_memory_usage_percent", "主机内存使用率");
        METRIC_NAME_MAP.put("host_memory_used_bytes", "主机内存使用量");
        METRIC_NAME_MAP.put("host_network_receive_speed_bps", "主机网口数据接收速度");
        METRIC_NAME_MAP.put("host_network_transmit_speed_bps", "主机网口数据发送速度");
        METRIC_NAME_MAP.put("host_network_receive_packets", "主机网口收包数量");
        METRIC_NAME_MAP.put("host_network_transmit_packets", "主机网口发包数量");
        METRIC_NAME_MAP.put("host_network_receive_bytes", "主机网口数据接收总量");
        METRIC_NAME_MAP.put("host_network_transmit_bytes", "主机网口数据发送总量");
        // 需要特殊处理 主机 hostname instance job role
        METRIC_NAME_MAP.put("host_memory_swap_size_bytes", "主机内存交换空间");
        METRIC_NAME_MAP.put("host_service_cpu_usage_percent", "主机服务系统CPU使用率%");
        // 新增
        METRIC_NAME_MAP.put("zbs_chunk_avg_readwrite_size_bytes", "主机I/O总平均块大小");
        METRIC_NAME_MAP.put("zbs_chunk_avg_read_size_bytes", "主机I/O读取平均块大小");
        METRIC_NAME_MAP.put("zbs_chunk_avg_write_size_bytes", "主机I/O写入平均块大小");
        METRIC_NAME_MAP.put("zbs_chunk_readwrite_cache_hit_ratio", "主机总缓存命中率");
        METRIC_NAME_MAP.put("zbs_chunk_read_cache_hit_ratio", "主机读取缓存命中率");
        METRIC_NAME_MAP.put("zbs_chunk_write_cache_hit_ratio", "主机写入缓存命中率");
        METRIC_NAME_MAP.put("elf_host_vcpus_provisioned_running", "主机分配给运行中(含暂停)虚拟机的逻辑核数");
        METRIC_NAME_MAP.put("elf_host_memory_provisioned_running_bytes", "主机分配给运行中(含暂停)虚拟机的内存大小");
        METRIC_NAME_MAP.put("host_cpu_temperature_celsius", "主机CPU温度");
        METRIC_NAME_MAP.put("host_memory_swap_in_speed_bps", "主机内存交换速率-Swapin");
        METRIC_NAME_MAP.put("host_memory_swap_out_speed_bps", "主机内存交换速率-SwapOut");
        METRIC_NAME_MAP.put("host_memory_swap_used_bytes", "主机已使用交换空间容量");
        METRIC_NAME_MAP.put("host_memory_swap_usage_percent", "主机交换空间使用率");
        METRIC_NAME_MAP.put("host_service_shared_memory_bytes", "主机系统服务共享内存使用");
        METRIC_NAME_MAP.put("host_service_resident_memory_bytes", "主机系统服务常驻内存使用");
        METRIC_NAME_MAP.put("host_network_ping_packet_loss_percent", "主机管理网络网口之间丢包率%");

        // 虚拟机
        METRIC_NAME_MAP.put("elf_vm_cpu_overall_usage_percent", "虚拟机CPU使用率%");
        METRIC_NAME_MAP.put("elf_vm_cpu_overall_used_hz", "虚拟机CPU使用频率");
        METRIC_NAME_MAP.put("elf_vm_memory_usage_percent", "虚拟机内存使用率%");
        METRIC_NAME_MAP.put("elf_vm_memory_used_bytes", "虚拟机内存使用量");
        METRIC_NAME_MAP.put("elf_vm_disk_overall_readwrite_iops", "虚拟机总IOPS");
        METRIC_NAME_MAP.put("elf_vm_disk_overall_read_iops", "虚拟机读IOPS");
        METRIC_NAME_MAP.put("elf_vm_disk_overall_write_iops", "虚拟机写IOPS");
        METRIC_NAME_MAP.put("elf_vm_disk_overall_readwrite_speed_bps", "虚拟机I/O总带宽");
        METRIC_NAME_MAP.put("elf_vm_disk_overall_read_speed_bps", "虚拟机I/O读取带宽");
        METRIC_NAME_MAP.put("elf_vm_disk_overall_write_speed_bps", "虚拟机I/O写入带宽");
        METRIC_NAME_MAP.put("elf_vm_disk_overall_avg_readwrite_latency_ns", "虚拟机I/O总延时");
        METRIC_NAME_MAP.put("elf_vm_disk_overall_avg_read_latency_ns", "虚拟机I/O平均读取延时");
        METRIC_NAME_MAP.put("elf_vm_disk_overall_avg_write_latency_ns", "虚拟机I/O平均写入延时");
        METRIC_NAME_MAP.put("elf_vm_disk_overall_avg_readwrite_size_bytes", "虚拟机I/O平均块大小-总");
        METRIC_NAME_MAP.put("elf_vm_disk_overall_avg_read_size_bytes", "虚拟机I/O平均块大小-读");
        METRIC_NAME_MAP.put("elf_vm_disk_overall_avg_write_size_bytes", "虚拟机I/O平均块大小-写");
        METRIC_NAME_MAP.put("elf_vm_network_transmit_packets", "虚拟机网口发包总量");
        METRIC_NAME_MAP.put("elf_vm_network_receive_packets", "虚拟机网口收包总量");
        METRIC_NAME_MAP.put("elf_vm_network_receive_errors", "虚拟机网口收包Error总量");
        METRIC_NAME_MAP.put("elf_vm_network_transmit_errors", "虚拟机网口发包Error总量");
        METRIC_NAME_MAP.put("elf_vm_network_receive_speed_bps", "虚拟机网口数据接收速度");
        METRIC_NAME_MAP.put("elf_vm_network_transmit_speed_bps", "虚拟机网口数据发送速度");
        METRIC_NAME_MAP.put("elf_vm_network_receive_bytes", "虚拟机网口数据接收总量");
        METRIC_NAME_MAP.put("elf_vm_network_transmit_bytes", "虚拟机网口数据发送总量");
        // 新增
        METRIC_NAME_MAP.put("elf_vm_network_receive_drop", "虚拟机网口收包中Dropped的包");
        METRIC_NAME_MAP.put("elf_vm_network_transmit_drop", "虚拟机网口发包中Dropped的包");

        // 集群   cluster: [{local_id: "030e4cbb-546b-4ec2-a173-6d463c9c1746"}]
        METRIC_NAME_MAP.put("cluster_cpu_usage_percent", "集群CPU使用率%");
        METRIC_NAME_MAP.put("cluster_cpu_used_hz", "集群CPU使用频率Hz");
        METRIC_NAME_MAP.put("cluster_memory_used_bytes", "集群内存使用量bytes");
        METRIC_NAME_MAP.put("zbs_cluster_readwrite_iops", "集群IOPS总读写");
        METRIC_NAME_MAP.put("zbs_cluster_read_iops", "集群IOPS读");
        METRIC_NAME_MAP.put("zbs_cluster_write_iops", "集群IOPS写");
        METRIC_NAME_MAP.put("zbs_cluster_readwrite_speed_bps", "集群I/O总带宽");
        METRIC_NAME_MAP.put("zbs_cluster_read_speed_bps", "集群I/O读带宽");
        METRIC_NAME_MAP.put("zbs_cluster_write_speed_bps", "集群I/O写带宽");
        METRIC_NAME_MAP.put("cluster_hdd_overall_size_bytes", "集群总HDD容量");
        METRIC_NAME_MAP.put("cluster_ssd_overall_size_bytes", "集群总SSD容量");
        METRIC_NAME_MAP.put("zbs_cluster_readwrite_cache_hit_ratio", "集群缓存总命中率");
        METRIC_NAME_MAP.put("zbs_cluster_write_cache_hit_ratio", "集群缓存写命中率");
        METRIC_NAME_MAP.put("zbs_cluster_read_cache_hit_ratio", "集群缓存读命中率");

        // 物理盘
        METRIC_NAME_MAP.put("host_disk_readwrite_iops", "物理盘总IOPS");
        METRIC_NAME_MAP.put("host_disk_read_iops", "物理盘读取IOPS");
        METRIC_NAME_MAP.put("host_disk_write_iops", "物理盘写入IOPS");
        METRIC_NAME_MAP.put("host_disk_readwrite_speed_bps", "物理盘I/O总带宽");
        METRIC_NAME_MAP.put("host_disk_read_speed_bps", "物理盘I/O读取带宽");
        METRIC_NAME_MAP.put("host_disk_write_speed_bps", "物理盘I/O写入带宽");
        METRIC_NAME_MAP.put("host_disk_avg_readwrite_latency_ns", "物理盘I/O总延时");
        METRIC_NAME_MAP.put("host_disk_avg_read_latency_ns", "物理盘I/O平均读取延时");
        METRIC_NAME_MAP.put("host_disk_avg_write_latency_ns", "物理盘I/O平均写入延时");
        METRIC_NAME_MAP.put("host_disk_avg_readwrite_size_bytes", "物理盘I/O总块大小");
        METRIC_NAME_MAP.put("host_disk_avg_read_size_bytes", "物理盘I/O平均读取大小");
        METRIC_NAME_MAP.put("host_disk_avg_write_size_bytes", "物理盘I/O平均写入大小");
        METRIC_NAME_MAP.put("host_disk_utilization_percent", "物理盘磁盘活动时间百分比Utilization");

        // 已挂载虚拟卷  labels: {vm_volume: [{volume_local_id: "38d1cb00-5c6e-44ba-8df0-d402413c80b2"}]}
        METRIC_NAME_MAP.put("elf_vm_disk_readwrite_iops", "已挂载虚拟卷总IOPS");
        METRIC_NAME_MAP.put("elf_vm_disk_read_iops", "已挂载虚拟卷读IOPS");
        METRIC_NAME_MAP.put("elf_vm_disk_write_iops", "已挂载虚拟卷写IOPS");
        METRIC_NAME_MAP.put("elf_vm_disk_readwrite_speed_bps", "已挂载虚拟卷总I/O带宽");
        METRIC_NAME_MAP.put("elf_vm_disk_read_speed_bps", "已挂载虚拟卷读I/O带宽");
        METRIC_NAME_MAP.put("elf_vm_disk_write_speed_bps", "已挂载虚拟卷写I/O带宽");
        METRIC_NAME_MAP.put("elf_vm_disk_avg_readwrite_latency_ns", "已挂载虚拟卷总I/O延时");
        METRIC_NAME_MAP.put("elf_vm_disk_avg_read_latency_ns", "已挂载虚拟卷读I/O延时");
        METRIC_NAME_MAP.put("elf_vm_disk_avg_write_latency_ns", "已挂载虚拟卷写I/O延时");
        METRIC_NAME_MAP.put("elf_vm_disk_avg_readwrite_size_bytes", "已挂载虚拟卷I/O平均块大小-总");
        METRIC_NAME_MAP.put("elf_vm_disk_avg_read_size_bytes", "已挂载虚拟卷I/O平均块大小-读");
        METRIC_NAME_MAP.put("elf_vm_disk_avg_write_size_bytes", "已挂载虚拟卷I/O平均块大小-写");

        // ISCSL LUN  zbs_volume_readwrite_iops
        METRIC_NAME_MAP.put("zbs_volume_readwrite_iops", "总IOPS");
        METRIC_NAME_MAP.put("zbs_volume_read_iops", "读取IOPS");
        METRIC_NAME_MAP.put("zbs_volume_write_iops", "写入IOPS");
        METRIC_NAME_MAP.put("zbs_volume_readwrite_speed_bps", "I/O总带宽");
        METRIC_NAME_MAP.put("zbs_volume_read_speed_bps", "I/O读取带宽");
        METRIC_NAME_MAP.put("zbs_volume_write_speed_bps", "I/O写入带宽");
        METRIC_NAME_MAP.put("zbs_volume_readwrite_latency_ns", "I/O平均延时-总");
        METRIC_NAME_MAP.put("zbs_volume_read_latency_ns", "I/O平均延时-读");
        METRIC_NAME_MAP.put("zbs_volume_write_latency_ns", "I/O平均延时-写");

        // 集群磁盘监控指标
        METRIC_NAME_MAP.put("zbs_cluster_data_capacity_bytes", "集群总数据容量");
        METRIC_NAME_MAP.put("zbs_cluster_used_data_space_bytes", "集群已使用所有数据总量");
        METRIC_NAME_MAP.put("zbs_chunk_valid_data_space_bytes", "集群中宿主机有效数据空间");
        METRIC_NAME_MAP.put("zbs_chunk_used_data_space_bytes", "集群中宿主机使用数据空间");
    }

    @Override
    protected Map<String, Object> getCache() {
        return this.cache;
    }

    @Value("${collector.smartx.enable:true}")
    boolean enable;

    @Async
    @Override
    @Scheduled(initialDelay = 8000L, fixedRate = 180000L)
    public void scheduledCollect() {
        if (!currentService.isLeader()) {
            logger.info(this.getClass().getName() + "当前节点是否是主节点:" + currentService.isLeader());
            return;
        }
        List<MetricFamilySamples> mfs = new ArrayList<>();
        Date now = new Date();
        logger.info("collect smartx metrics start...");
        try {
            // 获取bocloudResult对象
            BocloudResult bocloudResult = cmpCloudVendorService.list(CloudProvider.SMARTX);
            if (bocloudResult.isFailed()) {
                logger.error("get cloud vendor list failed , error:{}", bocloudResult.getMessage());
                return;
            }
            JSONObject data = (JSONObject) bocloudResult.getData();
            List<CloudVendor> listCloudVendor = data.getJSONArray(Common.ROWS).toJavaList(CloudVendor.class);
            Map<String, String> metricMap = new HashMap<>(16);
            // 放入指标名称等信息
            metricMap.putAll(METRIC_NAME_MAP);

            for (CloudVendor cloudVendor : listCloudVendor) {
                if (StringUtils.isBlank(cloudVendor.getAuthentication())) {
                    continue;
                }

                // 获取vendorId
                String vendorId = cloudVendor.getId().toString();

                String authentication = cloudVendor.getAuthentication();
                SmartAuthentication smartAuthentication = JSONArray.parseObject(authentication, SmartAuthentication.class);
                // API调用地址
                String address = smartAuthentication.getAddress();
                // 账号
                String username = smartAuthentication.getAccount();
                // 密码
                String password = smartAuthentication.getPassword();
                // 密码解密
                password = new DESEncrptor().decrypt(password.trim(), "BocloudCMP");

                // 通过post请求 获取到token信息
                SmartCommon smartCommon = new SmartCommon();
                // header固定写法
                Map<String, Object> header = new HashMap<>();
                header.put("Content-Type", "application/json");
                // params中为请求body信息 放入用户名和密码
                Map<String, Object> params = new HashMap<>();
                params.put("username", username);
                params.put("password", password);
                SmartxDatapoints datapoints = smartCommon.post(address+"/api/v3/sessions", header, params);
                String token = datapoints.getToken();

                // 直接调用监控API 获取监控数据
                for (String metricName : metricMap.keySet()) {
                    Map<String, Object> metricHeader = new HashMap<>();
                    metricHeader.put("Grpc-Metadata-Token", token);

                    // 调用API查询监控数据
                    String api = "%s/api/v3/metrics:query?query=%s";
                    String url = String.format(api, address, metricName);
                    // 获取监控指标集合
                    Result result = smartCommon.get(url, metricHeader, params);
                    JSONObject jsonObject = JSONObject.parseObject(result.getMessage());
                    // 获取到每一条监控数据
                    List<SmartMonitorResult> smartMonitorResults = jsonObject.getJSONObject("vector").getJSONArray("samples").toJavaList(SmartMonitorResult.class);
                    if (smartMonitorResults==null){
                        continue;
                    }
                    for (SmartMonitorResult smartMonitorResult : smartMonitorResults) {
                        // 指标采集
                        GaugeMetricFamily gauge = null;
                        // 集群 instance job
                        if ("cluster_cpu_usage_percent".equals(metricName) || "cluster_cpu_used_hz".equals(metricName) || "cluster_memory_used_bytes".equals(metricName)
                                || "zbs_cluster_readwrite_iops".equals(metricName) || "zbs_cluster_read_iops".equals(metricName) || "zbs_cluster_write_iops".equals(metricName)
                                || "zbs_cluster_readwrite_speed_bps".equals(metricName) || "zbs_cluster_read_speed_bps".equals(metricName) || "zbs_cluster_write_speed_bps".equals(metricName)
                                || "cluster_hdd_overall_size_bytes".equals(metricName) || "cluster_ssd_overall_size_bytes".equals(metricName) || "zbs_cluster_readwrite_cache_hit_ratio".equals(metricName)
                                || "zbs_cluster_write_cache_hit_ratio".equals(metricName) || "zbs_cluster_read_cache_hit_ratio".equals(metricName)){
                            gauge = new GaugeMetricFamily
                                    (PREFIX + metricName.toLowerCase(), "help", Arrays.asList("instance","job"));
                            String instance = smartMonitorResult.getLabels().getInstance();
                            String job = smartMonitorResult.getLabels().getJob();
                            Double monitorValue = smartMonitorResult.getPoint().getV() == null ? 0 : smartMonitorResult.getPoint().getV();
                            gauge.addMetric(Arrays.asList(instance,job), monitorValue);

                            // 主机 特殊处理
                        }else if ("host_network_ping_packet_loss_percent".equals(metricName)){
                            gauge = new GaugeMetricFamily
                                    // (PREFIX + metricName.toLowerCase(), "help", Arrays.asList("instance","job","host"));
                                    (PREFIX + metricName.toLowerCase(), "help", Arrays.asList("instance","job","name","vendorId"));         // 添加vendorId指标
                            String instance = smartMonitorResult.getLabels().getInstance();
                            String job = smartMonitorResult.getLabels().getJob();
                            String host = smartMonitorResult.getLabels().getToHostName();
                            Double monitorValue = smartMonitorResult.getPoint().getV() == null ? 0 : smartMonitorResult.getPoint().getV();
                            gauge.addMetric(Arrays.asList(instance,job,host,vendorId), monitorValue);

                            // 主机 instance job name
                        }else if ("zbs_chunk_readwrite_iops".equals(metricName) || "zbs_chunk_read_iops".equals(metricName) || "zbs_chunk_write_iops".equals(metricName)
                                || "zbs_chunk_readwrite_speed_bps".equals(metricName) || "zbs_chunk_read_speed_bps".equals(metricName) || "zbs_chunk_write_speed_bps".equals(metricName)
                                || "zbs_chunk_avg_readwrite_latency_ns".equals(metricName) || "zbs_chunk_avg_read_latency_ns".equals(metricName) || "zbs_chunk_avg_write_latency_ns".equals(metricName)
                                || "zbs_chunk_avg_readwrite_size_bytes".equals(metricName) || "zbs_chunk_avg_read_size_bytes".equals(metricName) || "zbs_chunk_avg_write_size_bytes".equals(metricName)
                                || "zbs_chunk_readwrite_cache_hit_ratio".equals(metricName) || "zbs_chunk_read_cache_hit_ratio".equals(metricName) || "zbs_chunk_write_cache_hit_ratio".equals(metricName)
                                || "zbs_chunk_cache_capacity_bytes".equals(metricName) || "zbs_chunk_data_capacity_bytes".equals(metricName)){
                            gauge = new GaugeMetricFamily
                                    (PREFIX + metricName.toLowerCase(), "help", Arrays.asList("instance","job","name","vendorId"));         // 添加vendorId指标
                            String instance = smartMonitorResult.getLabels().getInstance();
                            String job = smartMonitorResult.getLabels().getJob();
                            String name = smartMonitorResult.getLabels().getName();
                            Double monitorValue = smartMonitorResult.getPoint().getV() == null ? 0 : smartMonitorResult.getPoint().getV();
                            gauge.addMetric(Arrays.asList(instance,job,name,vendorId), monitorValue);

                            // 主机 hostname instance job role
                        }else if ("elf_host_vcpus_provisioned_running".equals(metricName) || "elf_host_memory_provisioned_running_bytes".equals(metricName) || "host_cpu_temperature_celsius".equals(metricName)
                                || "host_memory_swap_in_speed_bps".equals(metricName) || "host_memory_swap_out_speed_bps".equals(metricName) || "host_memory_swap_used_bytes".equals(metricName)
                                || "host_memory_swap_usage_percent".equals(metricName) || "host_service_shared_memory_bytes".equals(metricName) || "host_service_resident_memory_bytes".equals(metricName)){
                            // 主机统一配置hostname -> name 方便Prometheus采集
                            gauge = new GaugeMetricFamily
                                    // (PREFIX + metricName.toLowerCase(), "help", Arrays.asList("instance","job","hostname","role"));
                                    (PREFIX + metricName.toLowerCase(), "help", Arrays.asList("instance","job","name","role","vendorId"));         // 添加vendorId指标
                            String instance = smartMonitorResult.getLabels().getInstance();
                            String hostName = smartMonitorResult.getLabels().getHostName();
                            String job = smartMonitorResult.getLabels().getJob();
                            String role = smartMonitorResult.getLabels().getRole();
                            Double monitorValue = smartMonitorResult.getPoint().getV() == null ? 0 : smartMonitorResult.getPoint().getV();
                            gauge.addMetric(Arrays.asList(instance,job,hostName,role,vendorId), monitorValue);

                            // 主机 device hostname job role instance
                        }else if ("host_ssd_overall_size_bytes".equals(metricName) || "host_hdd_overall_size_bytes".equals(metricName) || "host_cpu_overall_usage_percent".equals(metricName)
                                || "elf_host_vcpus_provisioned".equals(metricName) || "elf_host_memory_provisioned_bytes".equals(metricName) || "host_cpu_overall_used_hz".equals(metricName)
                                || "host_cpu_overall_1m_avg_load".equals(metricName) || "host_cpu_overall_5m_avg_load".equals(metricName) || "host_cpu_overall_15m_avg_load".equals(metricName)
                                || "host_memory_usage_percent".equals(metricName) || "host_memory_used_bytes".equals(metricName)){
                            // 主机统一配置hostname -> name 方便Prometheus采集
                            gauge = new GaugeMetricFamily
                                    // (PREFIX + metricName.toLowerCase(), "help", Arrays.asList("instance","job","hostname"));
                                    (PREFIX + metricName.toLowerCase(), "help", Arrays.asList("instance","job","name","vendorId"));         // 添加vendorId指标
                            String instance = smartMonitorResult.getLabels().getInstance();
                            String hostName = smartMonitorResult.getLabels().getHostName();
                            String job = smartMonitorResult.getLabels().getJob();
                            Double monitorValue = smartMonitorResult.getPoint().getV() == null ? 0 : smartMonitorResult.getPoint().getV();
                            gauge.addMetric(Arrays.asList(instance,job,hostName,vendorId), monitorValue);

                            // 主机 device hostname job role instance
                            // todo 需要进行特殊处理 因为有device标签
                        }else if ("host_network_receive_speed_bps".equals(metricName) || "host_network_transmit_speed_bps".equals(metricName) || "host_network_receive_packets".equals(metricName)
                                || "host_network_transmit_packets".equals(metricName) || "host_network_receive_bytes".equals(metricName) || "host_network_transmit_bytes".equals(metricName)){
                            // 主机统一配置hostname -> name 方便Prometheus采集
                            gauge = new GaugeMetricFamily
                                    // (PREFIX + metricName.toLowerCase(), "help", Arrays.asList("instance","job","hostname","role","device"));
                                    (PREFIX + metricName.toLowerCase(), "help", Arrays.asList("instance","job","name","role","device","vendorId"));         // 添加vendorId指标
                            String instance = smartMonitorResult.getLabels().getInstance();
                            String hostName = smartMonitorResult.getLabels().getHostName();
                            String job = smartMonitorResult.getLabels().getJob();
                            String role = smartMonitorResult.getLabels().getRole();
                            String device = smartMonitorResult.getLabels().getDevice();
                            Double monitorValue = smartMonitorResult.getPoint().getV() == null ? 0 : smartMonitorResult.getPoint().getV();
                            gauge.addMetric(Arrays.asList(instance,job,hostName,role,device,vendorId), monitorValue);

                            // 主机 hostname instance job role
                        }else if("host_memory_swap_size_bytes".equals(metricName) || "host_service_cpu_usage_percent".equals(metricName)){
                            // 主机统一配置hostname -> name 方便Prometheus采集
                            gauge = new GaugeMetricFamily
                                    // (PREFIX + metricName.toLowerCase(), "help", Arrays.asList("instance","job","hostname","role"));
                                    (PREFIX + metricName.toLowerCase(), "help", Arrays.asList("instance","job","name","role","vendorId"));         // 添加vendorId指标
                            String instance = smartMonitorResult.getLabels().getInstance();
                            String hostName = smartMonitorResult.getLabels().getHostName();
                            String job = smartMonitorResult.getLabels().getJob();
                            String role = smartMonitorResult.getLabels().getRole();
                            Double monitorValue = smartMonitorResult.getPoint().getV() == null ? 0 : smartMonitorResult.getPoint().getV();
                            gauge.addMetric(Arrays.asList(instance,job,hostName,role,vendorId), monitorValue);

                            // 物理盘 device disk_model disk_type hostname instance role
                        }else if ("host_disk_readwrite_iops".equals(metricName) || "host_disk_read_iops".equals(metricName) || "host_disk_write_iops".equals(metricName)
                                || "host_disk_readwrite_speed_bps".equals(metricName) || "host_disk_read_speed_bps".equals(metricName) || "host_disk_write_speed_bps".equals(metricName)
                                || "host_disk_avg_readwrite_latency_ns".equals(metricName) || "host_disk_avg_read_latency_ns".equals(metricName) || "host_disk_avg_write_latency_ns".equals(metricName)
                                || "host_disk_avg_readwrite_size_bytes".equals(metricName) || "host_disk_avg_read_size_bytes".equals(metricName) || "host_disk_avg_write_size_bytes".equals(metricName)
                                || "host_disk_utilization_percent".equals(metricName)){
                            gauge = new GaugeMetricFamily
                                    (PREFIX + metricName.toLowerCase(), "help", Arrays.asList("instance","device","hostname","role","disk_model","disk_type"));
                            String instance = smartMonitorResult.getLabels().getInstance();
                            String hostName = smartMonitorResult.getLabels().getHostName();
                            String device = smartMonitorResult.getLabels().getDevice();
                            String diskModel = smartMonitorResult.getLabels().getDiskModel();
                            String diskType = smartMonitorResult.getLabels().getDiskType();
                            String role = smartMonitorResult.getLabels().getRole();
                            Double monitorValue = smartMonitorResult.getPoint().getV() == null ? 0 : smartMonitorResult.getPoint().getV();
                            gauge.addMetric(Arrays.asList(instance,device,hostName,role,diskModel,diskType), monitorValue);

                            // 虚拟机 vm instance job source
                        }else if ("elf_vm_cpu_overall_usage_percent".equals(metricName) || "elf_vm_cpu_overall_used_hz".equals(metricName)
                                || "elf_vm_memory_usage_percent".equals(metricName) || "elf_vm_memory_used_bytes".equals(metricName)){
                            gauge = new GaugeMetricFamily
                                    (PREFIX + metricName.toLowerCase(), "help", Arrays.asList("instance","job","vm","source","vendorId"));         // 添加vendorId指标
                            String instance = smartMonitorResult.getLabels().getInstance();
                            String job = smartMonitorResult.getLabels().getJob();
                            String vm = smartMonitorResult.getLabels().getVm();
                            String source = smartMonitorResult.getLabels().getSource();
                            Double monitorValue = smartMonitorResult.getPoint().getV() == null ? 0 : smartMonitorResult.getPoint().getV();
                            gauge.addMetric(Arrays.asList(instance,job,vm,source,vendorId), monitorValue);
                        }else if("elf_vm_disk_overall_readwrite_iops".equals(metricName) || "elf_vm_disk_overall_read_iops".equals(metricName)
                                || "elf_vm_disk_overall_write_iops".equals(metricName) || "elf_vm_disk_overall_readwrite_speed_bps".equals(metricName) || "elf_vm_disk_overall_read_speed_bps".equals(metricName)
                                || "elf_vm_disk_overall_write_speed_bps".equals(metricName) || "elf_vm_disk_overall_avg_readwrite_latency_ns".equals(metricName) || "elf_vm_disk_overall_avg_read_latency_ns".equals(metricName)
                                || "elf_vm_disk_overall_avg_write_latency_ns".equals(metricName) || "elf_vm_disk_overall_avg_readwrite_size_bytes".equals(metricName) || "elf_vm_disk_overall_avg_read_size_bytes".equals(metricName)
                                || "elf_vm_disk_overall_avg_write_size_bytes".equals(metricName) || "elf_vm_network_receive_drop".equals(metricName) || "elf_vm_network_transmit_drop".equals(metricName)){

                            gauge = new GaugeMetricFamily
                                    (PREFIX + metricName.toLowerCase(), "help", Arrays.asList("instance","job","vm","vendorId"));      // 添加vendorId指标
                            String instance = smartMonitorResult.getLabels().getInstance();
                            String job = smartMonitorResult.getLabels().getJob();
                            String vm = smartMonitorResult.getLabels().getVm();
                            Double monitorValue = smartMonitorResult.getPoint().getV() == null ? 0 : smartMonitorResult.getPoint().getV();
                            gauge.addMetric(Arrays.asList(instance,job,vm,vendorId), monitorValue);

                            // 虚拟机 mac vm instance job
                        }else if ("elf_vm_network_transmit_packets".equals(metricName) || "elf_vm_network_receive_packets".equals(metricName) || "elf_vm_network_receive_errors".equals(metricName)
                                || "elf_vm_network_transmit_errors".equals(metricName) || "elf_vm_network_receive_speed_bps".equals(metricName) || "elf_vm_network_transmit_speed_bps".equals(metricName)
                                || "elf_vm_network_receive_bytes".equals(metricName) || "elf_vm_network_transmit_bytes".equals(metricName)){
                            gauge = new GaugeMetricFamily
                                    (PREFIX + metricName.toLowerCase(), "help", Arrays.asList("instance","job","vm","mac","vendorId"));         // 添加vendorId指标
                            String instance = smartMonitorResult.getLabels().getInstance();
                            String job = smartMonitorResult.getLabels().getJob();
                            String vm = smartMonitorResult.getLabels().getVm();
                            String mac = smartMonitorResult.getLabels().getMac();
                            Double monitorValue = smartMonitorResult.getPoint().getV() == null ? 0 : smartMonitorResult.getPoint().getV();
                            gauge.addMetric(Arrays.asList(instance,job,vm,mac,vendorId), monitorValue);

                            // 已挂载虚拟卷 vm volume instance job
                        }else if ("elf_vm_disk_readwrite_iops".equals(metricName) || "elf_vm_disk_read_iops".equals(metricName) || "elf_vm_disk_write_iops".equals(metricName)
                                || "elf_vm_disk_readwrite_speed_bps".equals(metricName) || "elf_vm_disk_read_speed_bps".equals(metricName) || "elf_vm_disk_write_speed_bps".equals(metricName)
                                || "elf_vm_disk_avg_readwrite_latency_ns".equals(metricName) || "elf_vm_disk_avg_read_latency_ns".equals(metricName) || "elf_vm_disk_avg_write_latency_ns".equals(metricName)
                                || "elf_vm_disk_avg_readwrite_size_bytes".equals(metricName) || "elf_vm_disk_avg_read_size_bytes".equals(metricName) || "elf_vm_disk_avg_write_size_bytes".equals(metricName)){
                            gauge = new GaugeMetricFamily
                                    (PREFIX + metricName.toLowerCase(), "help", Arrays.asList("instance","job","vm","volume"));
                            String instance = smartMonitorResult.getLabels().getInstance();
                            String job = smartMonitorResult.getLabels().getJob();
                            String vm = smartMonitorResult.getLabels().getVm();
                            String volume = smartMonitorResult.getLabels().getVolume();
                            Double monitorValue = smartMonitorResult.getPoint().getV() == null ? 0 : smartMonitorResult.getPoint().getV();
                            gauge.addMetric(Arrays.asList(instance,job,vm,volume), monitorValue);

                            // ISCSL LUN  volume instance job pool
                        }else if ("zbs_volume_readwrite_iops".equals(metricName) || "zbs_volume_read_iops".equals(metricName) || "zbs_volume_write_iops".equals(metricName)
                                || "zbs_volume_readwrite_speed_bps".equals(metricName) || "zbs_volume_read_speed_bps".equals(metricName) || "zbs_volume_write_speed_bps".equals(metricName)
                                || "zbs_volume_readwrite_latency_ns".equals(metricName) || "zbs_volume_read_latency_ns".equals(metricName) || "zbs_volume_write_latency_ns".equals(metricName)){
                            gauge = new GaugeMetricFamily
                                    (PREFIX + metricName.toLowerCase(), "help", Arrays.asList("instance","job","pool","volume"));
                            String instance = smartMonitorResult.getLabels().getInstance();
                            String job = smartMonitorResult.getLabels().getJob();
                            String pool = smartMonitorResult.getLabels().getPool();
                            String volume = smartMonitorResult.getLabels().getVolume();
                            Double monitorValue = smartMonitorResult.getPoint().getV() == null ? 0 : smartMonitorResult.getPoint().getV();
                            gauge.addMetric(Arrays.asList(instance,job,pool,volume), monitorValue);

                            // 集群磁盘总监控指标 __name__ _cluster instance job
                        }else if ("zbs_cluster_data_capacity_bytes".equals(metricName) || "zbs_cluster_used_data_space_bytes".equals(metricName)){
                            gauge = new GaugeMetricFamily
                                    (PREFIX + metricName.toLowerCase(), "help", Arrays.asList("instance","job","vendorId"));
                            String instance = smartMonitorResult.getLabels().getInstance();
                            String job = smartMonitorResult.getLabels().getJob();
                            Double monitorValue = smartMonitorResult.getPoint().getV() == null ? 0 : smartMonitorResult.getPoint().getV();
                            gauge.addMetric(Arrays.asList(instance,job,vendorId), monitorValue);

                            // 集群分区磁盘监控指标 __name__ _chunk instance job name
                        }else if ("zbs_chunk_valid_data_space_bytes".equals(metricName) || "zbs_chunk_used_data_space_bytes".equals(metricName)){
                            gauge = new GaugeMetricFamily
                                    (PREFIX + metricName.toLowerCase(), "help", Arrays.asList("instance","job","name","vendorId"));
                            String instance = smartMonitorResult.getLabels().getInstance();
                            String job = smartMonitorResult.getLabels().getJob();
                            String name = smartMonitorResult.getLabels().getName();
                            Double monitorValue = smartMonitorResult.getPoint().getV() == null ? 0 : smartMonitorResult.getPoint().getV();
                            gauge.addMetric(Arrays.asList(instance,job,name,vendorId), monitorValue);
                        }
                        mfs.add(gauge);
                    }
                }
            }
        } catch (Exception e) {
            logger.error("collect smartx response error ", e);
        }
        cache.put(TIME, new Date());
        cache.put(DATA, mfs);
        logger.info("collect smartx metrics end! use " + (System.currentTimeMillis() - now.getTime()) / 1000 + "s");
    }
}
