package com.bocloud.mcs.collector.smis;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.encrypt.AESEncrptor;
import com.bocloud.common.encrypt.DESEncrptor;
import com.bocloud.common.enums.BoCloudService;
import com.bocloud.ims.entity.CloudVendor;
import com.bocloud.service.core.RegistryService;
import io.prometheus.client.GaugeMetricFamily;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.client.methods.HttpEntityEnclosingRequestBase;
import org.apache.http.client.methods.HttpUriRequest;
import org.apache.http.util.Asserts;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.*;
import org.springframework.http.client.ClientHttpResponse;
import org.springframework.http.client.HttpComponentsClientHttpRequestFactory;
import org.springframework.web.client.ResponseErrorHandler;
import org.springframework.web.client.RestTemplate;

import java.io.IOException;
import java.net.URI;
import java.net.URL;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public abstract class SmisService {

    private final Logger logger = LoggerFactory.getLogger(SmisService.class);
    private final String smisUriCim = "/api/pontus/v1/cim";
    private final String AES_KEY = "BocloudCMPV587!!";
    private final String DES_KEY = "BocloudCMP";
    private final RegistryService registryService;
    protected final RestTemplate restTemplate;
    protected final List<String> labels = Arrays.asList("vendorId", "instanceUuid", "instanceName", "parentUuid", "parentName", "instanceType");

    protected SmisService(RegistryService registryService) {
        this.registryService = registryService;

        restTemplate = new RestTemplate();
        restTemplate.setRequestFactory(new HttpComponentsClientHttpRequestFactory() {
            @Override
            protected HttpUriRequest createHttpUriRequest(HttpMethod httpMethod, URI uri) {
                if (httpMethod.equals(HttpMethod.GET)) {
                    HttpEntityEnclosingRequestBase http = new HttpEntityEnclosingRequestBase() {
                        @Override
                        public String getMethod() {
                            return HttpMethod.GET.name();
                        }
                    };
                    http.setURI(uri);
                    return http;
                }
                return super.createHttpUriRequest(httpMethod, uri);
            }
        });

        restTemplate.setErrorHandler(new ResponseErrorHandler() {
            @Override
            public boolean hasError(ClientHttpResponse response) throws IOException {
                return false;
            }

            @Override
            public void handleError(ClientHttpResponse response) throws IOException {
            }
        });
    }

    protected String getPontusUrl(String uri) {
        String pontusHost = registryService.getService(BoCloudService.PONTUS);
        Asserts.check(StringUtils.isNotEmpty(pontusHost), "Pontus服务未找到");
        return "http://" + pontusHost + uri;
    }

    protected Map<String, Object> getSmisProviderParam(CloudVendor vendor) throws Exception {
        String address = JSONObject.parseObject(vendor.getAuthentication()).getString("address");
        String account = JSONObject.parseObject(vendor.getAuthentication()).getString("account");
        String password = JSONObject.parseObject(vendor.getAuthentication()).getString("password");
        String projectName = JSONObject.parseObject(vendor.getAuthentication()).getString("projectName");
        Map<String, Object> map = new HashMap<>();
        URL url = new URL(address);
        map.put("host", url.getHost());
        if (url.getPort() == -1) {
            if ("http".equals(url.getProtocol())) {
                map.put("port", 5989);
            } else if ("https".equals(url.getProtocol())) {
                map.put("port", 5989);
            }
        } else {
            map.put("port", url.getPort());
        }
        map.put("user", account);
        map.put("password", new AESEncrptor().encrypt(new DESEncrptor().decrypt(password.trim(), DES_KEY), AES_KEY));
        map.put("type", projectName);
        return map;
    }

    protected String smisCim(CloudVendor vendor, SmisCimModel cimModel) throws Exception {
        String url = getPontusUrl(smisUriCim);
        Map<String, Object> requestMap = new HashMap<>();
        requestMap.put("provider", getSmisProviderParam(vendor));
        requestMap.put("params", cimModel);
        String requestBody = JSON.toJSONString(requestMap);
        logger.debug("[GET] {} ---> {}", url, requestBody);
        HttpHeaders httpHeaders = new HttpHeaders();
        httpHeaders.setContentType(MediaType.APPLICATION_JSON);
        HttpEntity<String> httpEntity = new HttpEntity<>(requestBody, httpHeaders);
        ResponseEntity<String> response = restTemplate.exchange(url, HttpMethod.GET, httpEntity, String.class);
        String responseBody = SmisUtil.unicodeToString(response.getBody());
        if (response.getStatusCodeValue() != 200) {
            logger.error(String.format("[GET] %s <--- [%s] %s", url, response.getStatusCodeValue(), responseBody));
            return null;
        }
        logger.debug("[GET] {} <--- {}", url, SmisUtil.trimAny(responseBody));
        return responseBody;
    }

    public abstract List<GaugeMetricFamily> getStat(CloudVendor vendor) throws Exception;

}
