package com.bocloud.mcs.collector.tianyicloud;

import com.bocloud.common.utils.MapTools;
import org.apache.commons.codec.binary.Base64;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.crypto.Mac;
import javax.crypto.spec.SecretKeySpec;
import java.security.MessageDigest;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;

/**
 * @Package: PACKAGE_NAME
 * @ClassName: toMD5Base64
 * @Author: 胡文涛
 * @CreateTime: 2020/9/25 10:20
 * @Description:
 */
public class Util {

    private static Logger logger = LoggerFactory.getLogger(Util.class);

//    //平台类型，取值范围：2或3，传2表示2.0自营资源，传3表示3.0合营资源，该参数不需要加密。
//    public static String platform = "3";
    public static Map<String, Object> sendGet(String apiUrl, Map<String, Object> headers, String accessKey, String secretKey, String platform) {
        Map<String, Object> get = null;
        try {
            get = new HashMap<>();
            // 构造消息头
            get.put("Connection", "Keep-Alive");
            get.put("Accept", "application/json");
            get.put("accessKey", accessKey);

            String contentMD5 = toMD5Base64("");
            String date = new SimpleDateFormat("EEE, d MMM yyyy HH:mm:ss z", Locale.ENGLISH).format(new Date());

            get.put("contentMD5", contentMD5);
            get.put("requestDate", date);

            String data = contentMD5 + "\n" + date + "\n" + apiUrl;
            String hmac = calculateHMAC(secretKey, data);
            get.put("hmac", hmac);
            get.put("platform", platform);

            if (!MapTools.isEmpty(headers)) {
                for (String key : headers.keySet()) {
                    Object value = headers.get(key);
                    get.put(key, value);
                }
            }
        } catch (Exception e) {
            logger.error("连接异常：" + e.getMessage(), e);
        }
        return get;
    }


    //private static final String HMAC_SHA1_ALGORITHM = "Ztjy9wSh5ifcybpg02vhyQr8u0zH5ddb87x9FrCT";

    /**
     * 将输入字符串转换为MD5
     *
     * @param input 目标字符串
     * @return MD5加密后的字符串
     */
    public static String toMD5Base64(String input) {
        try {
            MessageDigest md = MessageDigest.getInstance("MD5");
            md.update(input.getBytes());
            byte[] digest = md.digest();
            String result = new String(Base64.encodeBase64(digest));
            return result;
        } catch (Exception e) {
            logger.error(e.getMessage(), e);
            return null;
        }
    }

    /**
     * 根据HmacSHA1算法生成HMAC信息摘要
     *
     * @param secret 密钥
     * @param data   消息输入
     * @return 信息摘要
     */
    public static String calculateHMAC(String secret, String data) {
        try {
            SecretKeySpec secretKeySpec = new SecretKeySpec(secret.getBytes(), "HmacSHA1");
            Mac mac = Mac.getInstance("HmacSHA1");
            mac.init(secretKeySpec);
            byte[] rawHmac = mac.doFinal(data.getBytes());
            String result = new String(Base64.encodeBase64(rawHmac));
            return result;
        } catch (Exception e) {
            logger.error(e.getMessage(), e);
            throw new IllegalArgumentException();
        }
    }
}
