package com.bocloud.mcs.controller;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.LogBean;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.LogCrawler;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.util.ArrayList;
import java.util.List;

/**
 * @author hufy
 */
@RestController
public class LogViewController {
    private static Logger logger = LoggerFactory.getLogger(LogViewController.class);

    /**
     * 获取服务日志的信息
     *
     * @param filepath 日志文件的本地路径
     * @param position 日志文件的起始位点
     * @return 日志内容
     */
    @RequestMapping("/api/mcs/v1/logview")
    public Result fetch(@RequestParam(value = "filepath", required = true) String filepath,
                        @RequestParam(value = "position", required = true) Long position,
                        @RequestParam(value = "presize", required = true) Long presize) {
        BocloudResult<LogBean> crawlResult = LogCrawler.crawl(filepath, position, presize);
        if (crawlResult.isSuccess()) {
            return new Result(true, JSONObject.toJSONString(crawlResult.getData()));
        }
        return new Result(false, crawlResult.getMessage());
    }

    /**
     * 历史日志打包下载
     *
     * @param filepath
     * @param begin
     * @param end
     * @param server
     * @return
     * @throws Exception
     */
    @RequestMapping("/api/mcs/v1/logzip")
    public JSONObject zip(@RequestParam(value = "filepath", required = true) String filepath,
                          @RequestParam(value = "begin", required = true) String begin,
                          @RequestParam(value = "end", required = true) String end,
                          @RequestParam(value = "server", required = true) String server) throws Exception {
        List<File> files = new ArrayList<File>();
        File file = new File(filepath);
        File[] tempList = file.listFiles();
        if (null != tempList && tempList.length > 0) {
            for (int i = 0; i < tempList.length; i++) {
                if (tempList[i].isFile()) {
                    files.add(tempList[i]);
                }
            }
        }

        List<File> list = new ArrayList<>();
        for (File newFile : files) {
            if (newFile.getName().contains(server)) {
                // 判断文件名长度是否有14位，所需的文件长度都大于14位
                if (newFile.getName().length() < 14) {
                    continue;
                }
                // 截取文件名字中的第4-14个字符 ，拿到日期
                String date = newFile.getName().substring(4, 14);
                if (date.compareTo(begin) >= 0 && date.compareTo(end) <= 0) {
                    list.add(newFile);
                }
            }
        }

        JSONObject dataObj = new JSONObject();
        byte[] data = null;
        // 遍历文件，将其写入二进制数组
        for (File newFile : list) {
            try (FileInputStream fis = new FileInputStream(newFile);
                 ByteArrayOutputStream bos = new ByteArrayOutputStream()) {
                byte[] buffer = new byte[1024];
                int len;
                while ((len = fis.read(buffer)) != -1) {
                    bos.write(buffer, 0, len);
                }
                data = bos.toByteArray();
                dataObj.put(newFile.getName(), data);
            } catch (Exception e) {
                logger.error("Get error message：", e);
            }
        }
        return dataObj;
    }
}
