package com.bocloud.mcs.controller;

import com.alibaba.fastjson.JSON;
import com.bocloud.common.utils.ListTool;
import com.bocloud.mcs.entity.PrometheusHttpConfigBean;
import com.bocloud.mcs.entity.PrometheusHttpLabel;
import com.bocloud.service.core.CurrentService;
import com.bocloud.service.core.RegistryService;
import com.bocloud.service.core.Service;
import com.bocloud.service.registry.ZooKeeperClient;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.*;

/**
 * Prometheus http服务发现暴漏
 * @author fanzhenxi
 * @version 5.6.0-SNAPSHOT
 * @date 2022/2/23 18:23
 */
@RestController
public class PrometheusController {
    private static Logger logger = LoggerFactory.getLogger(PrometheusController.class);

    @Autowired
    private Service service;
    @Autowired
    private RegistryService registry;
    @Autowired
    private ZooKeeperClient zooKeeperClient;
    @Autowired
    private CurrentService currentService;

    @GetMapping("/targets")
    public String targets(HttpServletRequest request, HttpServletResponse response) throws Exception {
        boolean leader = currentService.isLeader();
        try {
            List<PrometheusHttpConfigBean> prometheusHttpConfigBeans = new ArrayList<>();
            PrometheusHttpLabel label = new PrometheusHttpLabel("china", "metrics");
            List<String> servers = getServices(service);
            Set<String> targets = new HashSet<>();
            Optional.ofNullable(servers).orElse(new ArrayList<>()).forEach(server -> {
                server = server.replaceAll("http://", "").replaceAll("https://", "");
                targets.add(server);
            });
            PrometheusHttpConfigBean prometheusHttpConfigBean = new PrometheusHttpConfigBean(targets,label);
            prometheusHttpConfigBeans.add(prometheusHttpConfigBean);
            String s = JSON.toJSONString(prometheusHttpConfigBeans);
            return JSON.toJSONString(prometheusHttpConfigBeans);
        }catch (Exception e){
            logger.error("获取zookeeper列表中节点失败！");
            return null;
        }
    }

    /**
     * 获取服务对应实例地址
     *
     * @param service 服务
     * @return 实例地址集合
     */
    private List<String> getServices(Service service) {
        try {
            List<String> serviceInstances = registry.getServices(service);
            return ListTool.isEmpty(serviceInstances) ? new ArrayList<>() : serviceInstances;
        } catch (RuntimeException e) {
            logger.error("get services error:" + e.getMessage());
            return new ArrayList<>();
        }
    }
}
